--------------------------------------------------------------------------------
-- SubModule FanoutAddress
-- Created   12/15/2007 2:38:02 PM
--
-- Parameters:
-- I : address of current module
-- TOP: if high, modify address for each one of a 4-way downstream module
--      if low, modify address for each one of a downstream receiver
-- A: first downstream module/transceiver
-- B: second downstream module/transceiver
-- C: third downstream module/transceiver
-- D: fourth downstream module/transceiver
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity FanoutAddress is port
   (
     A         : out   std_logic_vector(31 downto 0);
     B         : out   std_logic_vector(31 downto 0);
     C         : out   std_logic_vector(31 downto 0);
     D         : out   std_logic_vector(31 downto 0);
     I         : in    std_logic_vector(31 downto 0);
     TOP       : in    std_logic
   );
end FanoutAddress;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of FanoutAddress is

-- Component Declarations

-- Signal Declarations
   signal depth          : std_logic_vector (2 downto 0);
   signal aa             : std_logic_vector (30 downto 0);
   signal bb             : std_logic_vector (30 downto 0);
   signal cc             : std_logic_vector (30 downto 0);
   signal dd             : std_logic_vector (30 downto 0);

begin
   -- determine address depth level
   depth <= I (30 downto 28);

   -- modify address
   modaddr: process (I, TOP, depth) is
   begin
      if TOP = '1' then
         -- increase address offset value and set downstream address
         case depth is
            -- invalid (fanouts nested too deep)
            when B"111" =>
               aa <= (others => '0');
               bb <= (others => '0');
               cc <= (others => '0');
               dd <= (others => '0');
            -- Level 1
            when B"000" =>
               aa <= B"001" & B"0000" & I(23 downto 0);
               bb <= B"001" & B"0100" & I(23 downto 0);
               cc <= B"001" & B"1000" & I(23 downto 0);
               dd <= B"001" & B"1100" & I(23 downto 0);
            -- Level 2
            when B"001" =>
               aa <= B"010" & I(27 downto 24) & B"0000" & I(19 downto 0);
               bb <= B"010" & I(27 downto 24) & B"0100" & I(19 downto 0);
               cc <= B"010" & I(27 downto 24) & B"1000" & I(19 downto 0);
               dd <= B"010" & I(27 downto 24) & B"1100" & I(19 downto 0);
            -- Level 3
            when B"010" =>
               aa <= B"011" & I(27 downto 20) & B"0000" & I(15 downto 0);
               bb <= B"011" & I(27 downto 20) & B"0100" & I(15 downto 0);
               cc <= B"011" & I(27 downto 20) & B"1000" & I(15 downto 0);
               dd <= B"011" & I(27 downto 20) & B"1100" & I(15 downto 0);
            -- Level 4
            when B"011" =>
               aa <= B"100" & I(27 downto 16) & B"0000" & I(11 downto 0);
               bb <= B"100" & I(27 downto 16) & B"0100" & I(11 downto 0);
               cc <= B"100" & I(27 downto 16) & B"1000" & I(11 downto 0);
               dd <= B"100" & I(27 downto 16) & B"1100" & I(11 downto 0);
            -- Level 5
            when B"100" =>
               aa <= B"101" & I(27 downto 12) & B"0000" & I(7 downto 0);
               bb <= B"101" & I(27 downto 12) & B"0100" & I(7 downto 0);
               cc <= B"101" & I(27 downto 12) & B"1000" & I(7 downto 0);
               dd <= B"101" & I(27 downto 12) & B"1100" & I(7 downto 0);
            -- Level 6
            when B"101" =>
               aa <= B"110" & I(27 downto 8) & B"0000" & I(3 downto 0);
               bb <= B"110" & I(27 downto 8) & B"0100" & I(3 downto 0);
               cc <= B"110" & I(27 downto 8) & B"1000" & I(3 downto 0);
               dd <= B"110" & I(27 downto 8) & B"1100" & I(3 downto 0);
            -- Level 7
            when others =>
               aa <= B"111" & I(27 downto 4) & B"0000";
               bb <= B"111" & I(27 downto 4) & B"0100";
               cc <= B"111" & I(27 downto 4) & B"1000";
               dd <= B"111" & I(27 downto 4) & B"1100";
         end case;
      else
         -- modify the 2 LSB of the downstream address
         case depth is
            -- invalid (only top node can be zero)
            when B"000" =>
               aa <= (others => '0');
               bb <= (others => '0');
               cc <= (others => '0');
               dd <= (others => '0');
            -- Level 1
            when B"001" =>
               aa <= I(30 downto 26) & B"00" & I(23 downto 0);
               bb <= I(30 downto 26) & B"01" & I(23 downto 0);
               cc <= I(30 downto 26) & B"10" & I(23 downto 0);
               dd <= I(30 downto 26) & B"11" & I(23 downto 0);
            -- Level 2
            when B"010" =>
               aa <= I(30 downto 22) & B"00" & I(19 downto 0);
               bb <= I(30 downto 22) & B"01" & I(19 downto 0);
               cc <= I(30 downto 22) & B"10" & I(19 downto 0);
               dd <= I(30 downto 22) & B"11" & I(19 downto 0);
            -- Level 3
            when B"011" =>
               aa <= I(30 downto 18) & B"00" & I(15 downto 0);
               bb <= I(30 downto 18) & B"01" & I(15 downto 0);
               cc <= I(30 downto 18) & B"10" & I(15 downto 0);
               dd <= I(30 downto 18) & B"11" & I(15 downto 0);
            -- Level 4
            when B"100" =>
               aa <= I(30 downto 14) & B"00" & I(11 downto 0);
               bb <= I(30 downto 14) & B"01" & I(11 downto 0);
               cc <= I(30 downto 14) & B"10" & I(11 downto 0);
               dd <= I(30 downto 14) & B"11" & I(11 downto 0);
            -- Level 5
            when B"101" =>
               aa <= I(30 downto 10) & B"00" & I(7 downto 0);
               bb <= I(30 downto 10) & B"01" & I(7 downto 0);
               cc <= I(30 downto 10) & B"10" & I(7 downto 0);
               dd <= I(30 downto 10) & B"11" & I(7 downto 0);
            -- Level 6
            when B"110" =>
               aa <= I(30 downto 6) & B"00" & I(3 downto 0);
               bb <= I(30 downto 6) & B"01" & I(3 downto 0);
               cc <= I(30 downto 6) & B"10" & I(3 downto 0);
               dd <= I(30 downto 6) & B"11" & I(3 downto 0);
            -- Level 7
            when others =>
               aa <= I(30 downto 2) & B"00";
               bb <= I(30 downto 2) & B"01";
               cc <= I(30 downto 2) & B"10";
               dd <= I(30 downto 2) & B"11";
         end case;
      end if;
   end process modaddr;

   -- assign output bits
   A(31) <= I(31);
   A(30 downto 0) <= aa;
   B(31) <= I(31);
   B(30 downto 0) <= bb;
   C(31) <= I(31);
   C(30 downto 0) <= cc;
   D(31) <= I(31);
   D(30 downto 0) <= dd;

end Structure;
--------------------------------------------------------------------------------
